package de.devloop.mavor.servlet;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.stream.Collectors;

import de.devloop.mavor.AuthenticatedServlet;
import jakarta.servlet.RequestDispatcher;
import jakarta.servlet.ServletException;
import jakarta.servlet.annotation.WebServlet;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

@WebServlet("/download")
public class Download extends AuthenticatedServlet {

    private static final String PARAMETER_SITE = "site";
    private static final String PARAMETER_GROUP_ID = "groupId";
    private static final String PARAMETER_ARTIFACT_ID = "artifactId";
    private static final String PARAMETER_VERSION = "version";
    private static final String PARAMETER_POM = "pom";
    private static final String PARAMETER_TYPE = "type";

    private static final String MAVEN_CMD = "/usr/bin/mvn";
    private static final String TEMP_DIR = "/home/damage/Temp/mavor";

    @Override
    protected void doAuthenticatedGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        // implement a lot of checks:
        // NPE, required parameters not given

        String type = req.getParameter(PARAMETER_TYPE);
        if (type.equals("artifact")) {
            String site = req.getParameter(PARAMETER_SITE);
            String groupId = req.getParameter(PARAMETER_GROUP_ID);
            String artifactId = req.getParameter(PARAMETER_ARTIFACT_ID);
            String version = req.getParameter(PARAMETER_VERSION);
            generatePomXmlByArtifact(site, groupId, artifactId, version);
        } else if (type.equals("pom")) {
            writePomXml(req.getParameter(PARAMETER_POM));
        } else {
            throw new ServletException("Unknown download type: " + type);
        }

        ExecutionResult executionResult;
        try {
            executionResult = executeMaven();
        } catch (InterruptedException e) {
            throw new ServletException("Running maven failed", e);
        }

        req.setAttribute("stdout", executionResult.stdout);
        req.setAttribute("stderr", executionResult.stderr);
        req.setAttribute("exitcode", executionResult.exitCode);

        RequestDispatcher view = req.getRequestDispatcher("/download.jsp");
        view.forward(req, resp);
    }

    private ExecutionResult executeMaven() throws IOException, InterruptedException {
        Process mvnProcess = Runtime.getRuntime().exec(MAVEN_CMD + " dependency:copy-dependencies -DoutputDirectory=" + TEMP_DIR, null, new File(TEMP_DIR));
        String stdout = new String(mvnProcess.getInputStream().readAllBytes(), StandardCharsets.UTF_8);
        String stderr = new String(mvnProcess.getErrorStream().readAllBytes(), StandardCharsets.UTF_8);

        ExecutionResult executionResult = new ExecutionResult();
        
        executionResult.exitCode = mvnProcess.waitFor();
        executionResult.stdout = stdout;
        executionResult.stderr = stderr;
        return executionResult;
    }

    private void generatePomXmlByArtifact(String site, String groupId, String artifactId, String version) throws IOException {
        String pomXml = getResourceFileAsString("pomstub.xml");
        pomXml = pomXml.replace("{site-id}", "foo");
        pomXml = pomXml.replace("{site-url}", site);
        pomXml = pomXml.replace("{group-id}", groupId);
        pomXml = pomXml.replace("{artifact-id}", artifactId);
        pomXml = pomXml.replace("{version}", version);
        writePomXml(pomXml);
    }

    private void writePomXml(String pomXml) throws IOException {
        File pomXmlFile = new File(TEMP_DIR + "/pom.xml");
        pomXmlFile.delete();
        FileWriter fileWriter = new FileWriter(pomXmlFile);
        fileWriter.write(pomXml);
        fileWriter.close();
    }

    /**
     * Reads given resource file as a string.
     *
     * @param fileName path to the resource file
     * @return the file's contents
     * @throws IOException if read fails for any reason
     */
    static String getResourceFileAsString(String fileName) throws IOException {
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        try (InputStream is = classLoader.getResourceAsStream(fileName)) {
            if (is == null) return null;
            try (InputStreamReader isr = new InputStreamReader(is);
                BufferedReader reader = new BufferedReader(isr)) {
                return reader.lines().collect(Collectors.joining(System.lineSeparator()));
            }
        }
    }

    private class ExecutionResult {
        public int exitCode;
        public String stdout;
        public String stderr;
    }
}